# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaPOS -- Pythonic Point of Sale System
#  Copyright © 2023 Lance Edgar
#
#  This file is part of WuttaPOS.
#
#  WuttaPOS is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaPOS is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaPOS.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
WuttaPOS - base lookup control
"""

import flet as ft

from .base import WuttaControl
from .keyboard import WuttaKeyboard


class WuttaLookup(WuttaControl):

    default_font_size = 40
    font_size = default_font_size * 0.8
    default_button_height_dlg = 80
    disabled_bgcolor = '#aaaaaa'

    def __init__(self, *args, **kwargs):
        self.initial_search = kwargs.pop('initial_search', None)
        self.on_select = kwargs.pop('on_select', None)
        self.on_cancel = kwargs.pop('on_cancel', None)
        super().__init__(*args, **kwargs)

        # track current selection
        self.selected_uuid = None
        self.selected_control = None

    def build(self):

        self.searchbox = ft.TextField("", text_size=self.font_size * 0.8,
                                      on_submit=self.lookup,
                                      autofocus=True,
                                      expand=True)

        self.search_results = ft.DataTable(
            columns=[ft.DataColumn(self.make_cell_text(text))
                     for text in self.get_results_columns()],
        )

        self.no_results = ft.Text("NO RESULTS", size=32, color='red',
                                  weight=ft.FontWeight.BOLD,
                                  visible=False)

        self.select_button = ft.Container(
            content=ft.Text("Select", size=self.font_size * 0.8),
            alignment=ft.alignment.center,
            height=self.default_button_height_dlg * 0.8,
            width=self.default_button_height_dlg * 1.3,
            border=ft.border.all(1, 'black'),
            border_radius=ft.border_radius.all(5),
            on_click=self.select_object,
            disabled=True,
            bgcolor=self.disabled_bgcolor,
        )

        return ft.Column(
            [
                ft.Row(
                    [
                        ft.Text("SEARCH FOR:"),
                        self.searchbox,
                        ft.Container(
                            content=ft.Text("Lookup", size=self.font_size * 0.8),
                            alignment=ft.alignment.center,
                            height=self.default_button_height_dlg * 0.8,
                            width=self.default_button_height_dlg * 1.3,
                            border=ft.border.all(1, 'black'),
                            border_radius=ft.border_radius.all(5),
                            on_click=self.lookup,
                            bgcolor='blue',
                        ),
                        ft.Container(
                            content=ft.Text("Reset", size=self.font_size * 0.8),
                            alignment=ft.alignment.center,
                            height=self.default_button_height_dlg * 0.8,
                            width=self.default_button_height_dlg * 1.3,
                            border=ft.border.all(1, 'black'),
                            border_radius=ft.border_radius.all(5),
                            on_click=self.reset,
                            bgcolor='yellow',
                        ),
                        ft.Container(
                            content=ft.Text("Cancel", size=self.font_size * 0.8),
                            alignment=ft.alignment.center,
                            height=self.default_button_height_dlg * 0.8,
                            width=self.default_button_height_dlg * 1.3,
                            border=ft.border.all(1, 'black'),
                            border_radius=ft.border_radius.all(5),
                            on_click=self.cancel,
                        ),
                    ],
                ),
                ft.Divider(),
                WuttaKeyboard(self.config, on_keypress=self.keypress,
                              on_long_backspace=self.long_backspace),
                ft.Divider(),
                ft.Row(
                    [
                        ft.Column(
                            [
                                self.search_results,
                                self.no_results,
                            ],
                            expand=True,
                        ),
                        self.select_button,
                    ],
                ),
            ],
        )

    def get_results_columns(self):
        raise NotImplementedError

    def did_mount(self):
        if self.initial_search:
            self.searchbox.value = self.initial_search
            self.initial_search = None # only do it once
            self.update()
            self.lookup()

    def make_cell_text(self, text):
        return ft.Text(text, size=32)

    def make_cell(self, text):
        return ft.DataCell(self.make_cell_text(text))

    def cancel(self, e):
        if self.on_cancel:
            self.on_cancel(e)

    def keypress(self, key):
        if key == '⏎':
            self.lookup()
        else:
            if key == '⌫':
                self.searchbox.value = self.searchbox.value[:-1]
            else:
                self.searchbox.value += key
            self.searchbox.focus()
            self.update()

    def long_backspace(self):
        self.searchbox.value = self.searchbox.value[:-10]
        self.searchbox.focus()
        self.update()

    def get_results(self, session, entry):
        raise NotImplementedError

    def make_result_row(self, obj):
        raise NotImplementedError

    def lookup(self, e=None):
        entry = self.searchbox.value
        if not entry:
            self.searchbox.focus()
            self.update()
            return

        session = self.app.make_session()
        results = self.get_results(session, entry)

        self.search_results.rows.clear()
        self.selected_uuid = None
        self.select_button.disabled = True
        self.select_button.bgcolor = self.disabled_bgcolor

        if results:
            for obj in results:
                self.search_results.rows.append(ft.DataRow(
                    cells=[self.make_cell(row)
                           for row in self.make_result_row(obj)],
                    on_select_changed=self.select_changed,
                    data={'uuid': obj['uuid']},
                ))
            self.no_results.visible = False

        else:
            self.no_results.value = f"NO RESULTS FOR: {entry}"
            self.no_results.visible = True

        self.searchbox.focus()
        self.update()

    def reset(self, e):
        self.searchbox.value = ""
        self.search_results.rows.clear()
        self.no_results.visible = False
        self.selected_uuid = None
        self.select_button.disabled = True
        self.select_button.bgcolor = self.disabled_bgcolor
        self.searchbox.focus()
        self.update()

    def select_changed(self, e):

        if e.data: # selected
            if self.selected_control:
                self.selected_control.color = None
            self.selected_uuid = e.control.data['uuid']
            self.selected_control = e.control
            self.selected_control.color = ft.colors.BLUE
            self.select_button.disabled = False
            self.select_button.bgcolor = 'blue'
        else:
            if self.selected_control:
                self.selected_control.color = None
            self.selected_control = None
            self.selected_uuid = None
            self.select_button.disabled = True
            self.select_button.bgcolor = self.disabled_bgcolor
            e.control.color = None

        self.update()

    def select_object(self, e):
        if not self.selected_uuid:
            raise RuntimeError("no record selected?")
        if self.on_select:
            self.on_select(self.selected_uuid)
